@@ -1,21 +1,25 @@
 /* -*- mode: c++ -*- */
 /* File : example.h -- Tests all string typemaps */
 
+#include <sys/time.h>
+#include <time.h>
+
 void takes_std_string( std::string in ) {
     cout << "takes_std_string( \"" << in << "\" );" << endl;
 }
 
 std::string gives_std_string() {
-    time_t t;
-    
-    return std::string( asctime( localtime( &t ) ) );
+    struct timeval tv;
+
+    gettimeofday(&tv, NULL);
+    return std::string( asctime( localtime( &tv.tv_sec ) ) );
 }
 
 void takes_char_ptr( char *p ) {
     cout << "takes_char_ptr( \"" << p << "\" );" << endl;
 }
 
-char *gives_char_ptr() {
+const char *gives_const_char_ptr() {
     return "foo";
 }
 
@@ -24,10 +28,10 @@ void takes_and_gives_std_string( std::string &inout ) {
     inout.insert( inout.end(), ']' );
 }
 
-void takes_and_gives_char_ptr( char *&ptr ) {
-    char *pout = strchr( ptr, '.' );
-    if( pout ) ptr = pout + 1;
-    else ptr = "foo";
+void takes_and_gives_const_char_ptr( const char *&inout ) {
+    const char *pout = strchr( inout, '.' );
+    if( pout ) inout = pout + 1;
+    else inout = "foo";
 }
 
 /*
