@@ -45,6 +45,12 @@ public class doxygen_parsing_runme {
     		" The struct comment \n" +
     		" \n" +
     		"");
+    wantedComments.put("doxygen_parsing.SomeStruct.setWidth(int)",
+    		"**immutable** image width in pixels \n" +
+    		"");
+    wantedComments.put("doxygen_parsing.SomeStruct.getWidth()",
+    		"**immutable** image width in pixels \n" +
+    		"");
     wantedComments.put("doxygen_parsing.doxygen_parsing.setSomeVar(int)",
     		" The var comment \n" +
     		" \n" +
@@ -126,6 +132,14 @@ public class doxygen_parsing_runme {
     wantedComments.put("doxygen_parsing.doxygen_parsingConstants.CONSTANT_VALUE",
     		"The constant comment \n" +
     		"");
+    wantedComments.put("doxygen_parsing.Foo1750.getA()",
+	        "");
+    wantedComments.put("doxygen_parsing.Foo1750.getB()",
+	        "");
+    wantedComments.put("doxygen_parsing.Foo1750.setA(int)",
+	        "");
+    wantedComments.put("doxygen_parsing.Foo1750.setB(int)",
+	        "");
     
     // and ask the parser to check comments for us
     System.exit(CommentParser.check(wantedComments));
