@@ -1,16 +1,17 @@
 <?php
 
 require "tests.php";
-require "director_exception.php";
 
-// No new functions
-check::functions(array(foo_ping,foo_pong,launder,bar_ping,bar_pong,bar_pang,returnalltypes_return_int,returnalltypes_return_double,returnalltypes_return_const_char_star,returnalltypes_return_std_string,returnalltypes_return_bar,returnalltypes_call_int,returnalltypes_call_double,returnalltypes_call_const_char_star,returnalltypes_call_std_string,returnalltypes_call_bar,is_python_builtin));
-// No new classes
-check::classes(array(director_exception,Foo,Exception1,Exception2,Base,Bar,ReturnAllTypes));
-// now new vars
+// New functions
+check::functions(array('launder'));
+// New classes
+check::classes(array('director_exception','Foo','Exception1','Exception2','Base','Bar','ReturnAllTypes'));
+// No new vars
 check::globals(array());
 
 class MyException extends Exception {
+  public $msg;
+
   function __construct($a, $b) {
     $this->msg = $a . $b;
   }
@@ -38,9 +39,7 @@ class MyFoo3 extends Foo {
 # MyFoo.pong().
 $ok = 0;
 $a = new MyFoo();
-# TODO: Currently we do not track the dynamic type of returned 
-# objects, so we skip the launder() call.
-#$b = director_exception::launder($a);
+$b = director_exception::launder($a);
 $b = $a;
 try {
   $b->pong();
@@ -124,4 +123,3 @@ try {
 }
 
 check::done();
-?>
