@@ -1,7 +1,6 @@
 <?php
 
 require "tests.php";
-require "primitive_ref.php";
 
 # A large long long number is too big, so PHP makes treats it as a double, but SWIG opts to return it as a string.
 # The conversion to double can lose precision so this isn't an exact comparison.
@@ -31,4 +30,3 @@ long_long_equal(ref_longlong(0x123456789ABCDEF0), 0x123456789ABCDEF0, "ref_longl
 long_long_equal(ref_ulonglong(0xF23456789ABCDEF0), 0xF23456789ABCDEF0, "ref_ulonglong failed");
 
 check::done();
-?>
