@@ -4,14 +4,14 @@
  * terms also apply to certain portions of SWIG. The full details of the SWIG
  * license and copyrights can be found in the LICENSE and COPYRIGHT files
  * included with the SWIG source code as distributed by the SWIG developers
- * and at http://www.swig.org/legal.html.
+ * and at https://www.swig.org/legal.html.
  *
  * utils.cxx
  *
  * Various utility functions.
  * ----------------------------------------------------------------------------- */
 
-#include <swigmod.h>
+#include "swigmod.h"
 
 int is_public(Node *n) {
   String *access = Getattr(n, "access");
